@@ -56,10 +56,8 @@
 #define NAUTILUS_ACTION_OPEN "Open"
 #define NAUTILUS_ACTION_OPEN_ALTERNATE "OpenAlternate"
 #define NAUTILUS_ACTION_OPEN_IN_NEW_TAB "OpenInNewTab"
-#define NAUTILUS_ACTION_OPEN_FOLDER_WINDOW "OpenFolderWindow"
 #define NAUTILUS_ACTION_LOCATION_OPEN_ALTERNATE "LocationOpenAlternate"
 #define NAUTILUS_ACTION_LOCATION_OPEN_IN_NEW_TAB "LocationOpenInNewTab"
-#define NAUTILUS_ACTION_LOCATION_OPEN_FOLDER_WINDOW "LocationOpenFolderWindow"
 #define NAUTILUS_ACTION_OTHER_APPLICATION1 "OtherApplication1"
 #define NAUTILUS_ACTION_OTHER_APPLICATION2 "OtherApplication2"
 #define NAUTILUS_ACTION_NEW_FOLDER "New Folder"
