@@ -30,20 +30,18 @@
 
 #include "nautilus-application.h"
 
-#include "file-manager/fm-desktop-icon-view.h"
-#include "file-manager/fm-icon-view.h"
-#include "file-manager/fm-list-view.h"
 #if ENABLE_EMPTY_VIEW
-#include "file-manager/fm-empty-view.h"
+#include "nautilus-empty-view.h"
 #endif /* ENABLE_EMPTY_VIEW */
 
-#include "nautilus-application-smclient.h"
+#include "nautilus-desktop-icon-view.h"
 #include "nautilus-desktop-window.h"
+#include "nautilus-icon-view.h"
 #include "nautilus-image-properties-page.h"
-#include "nautilus-navigation-window.h"
-#include "nautilus-navigation-window-slot.h"
+#include "nautilus-list-view.h"
+#include "nautilus-progress-ui-handler.h"
 #include "nautilus-self-check-functions.h"
-#include "nautilus-spatial-window.h"
+#include "nautilus-window.h"
 #include "nautilus-window-bookmarks.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-private.h"
@@ -58,6 +56,7 @@
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-signaller.h>
+#include <libnautilus-private/nautilus-ui-utilities.h>
 #include <libnautilus-private/nautilus-undo-manager.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
 
@@ -73,6 +72,7 @@
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
+#include <libnotify/notify.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
@@ -89,9 +89,6 @@ static NautilusApplication *singleton = NULL;
 /* Keeps track of all the desktop windows. */
 static GList *nautilus_application_desktop_windows;
 
-/* Keeps track of all the object windows */
-static GList *nautilus_application_spatial_window_list;
-
 /* The saving of the accelerator map was requested  */
 static gboolean save_of_accel_map_requested = FALSE;
 
@@ -105,11 +102,12 @@ static void     mount_added_callback              (GVolumeMonitor            *mo
 
 G_DEFINE_TYPE (NautilusApplication, nautilus_application, GTK_TYPE_APPLICATION);
 
-static GList *
-nautilus_application_get_spatial_window_list (void)
-{
-	return nautilus_application_spatial_window_list;
-}
+struct _NautilusApplicationPriv {
+	GVolumeMonitor *volume_monitor;
+	NautilusProgressUIHandler *progress_handler;
+
+	gboolean initialized;
+};
 
 static gboolean
 check_required_directories (NautilusApplication *application)
@@ -192,12 +190,10 @@ menu_provider_items_updated_handler (NautilusMenuProvider *provider, GtkWidget*
 static void
 menu_provider_init_callback (void)
 {
-        GList *items;
         GList *providers;
         GList *l;
 
         providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_MENU_PROVIDER);
-        items = NULL;
 
         for (l = providers; l != NULL; l = l->next) {
                 NautilusMenuProvider *provider = NAUTILUS_MENU_PROVIDER (l->data);
@@ -263,150 +259,6 @@ mark_desktop_files_trusted (void)
 	g_free (do_once_file);
 }
 
-#define CK_NAME       "org.freedesktop.ConsoleKit"
-#define CK_PATH       "/org/freedesktop/ConsoleKit"
-#define CK_INTERFACE  "org.freedesktop.ConsoleKit"
-
-static void
-ck_session_proxy_signal_cb (GDBusProxy *proxy,
-			    const char *sender_name,
-			    const char *signal_name,
-			    GVariant   *parameters,
-			    gpointer    user_data)
-{
-	NautilusApplication *application = user_data;
-
-	if (g_strcmp0 (signal_name, "ActiveChanged") == 0) {
-		g_variant_get (parameters, "(b)", &application->session_is_active);
-		DEBUG ("ConsoleKit session is active %d", application->session_is_active);
-	}
-}
-
-static void
-ck_call_is_active_cb (GDBusProxy   *proxy,
-		      GAsyncResult *result,
-		      gpointer      user_data)
-{
-	NautilusApplication *application = user_data;
-	GVariant *variant;
-	GError *error = NULL;
-
-	variant = g_dbus_proxy_call_finish (proxy, result, &error);
-
-	if (variant == NULL) {
-		g_warning ("Error when calling IsActive(): %s\n", error->message);
-		application->session_is_active = TRUE;
-
-		g_error_free (error);
-		return;
-	}
-
-	g_variant_get (variant, "(b)", &application->session_is_active);
-	DEBUG ("ConsoleKit session is active %d", application->session_is_active);
-
-	g_variant_unref (variant);
-}
-
-static void
-session_proxy_appeared (GObject       *source,
-                        GAsyncResult *res,
-                        gpointer      user_data)
-{
-	NautilusApplication *application = user_data;
-        GDBusProxy *proxy;
-	GError *error = NULL;
-
-        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-	if (error != NULL) {
-		g_warning ("Failed to get the current CK session: %s", error->message);
-		g_error_free (error);
-
-		application->session_is_active = TRUE;
-		return;
-	}
-
-	g_signal_connect (proxy, "g-signal",
-			  G_CALLBACK (ck_session_proxy_signal_cb),
-			  application);
-
-	g_dbus_proxy_call (proxy,
-			   "IsActive",
-			   g_variant_new ("()"),
-			   G_DBUS_CALL_FLAGS_NONE,
-			   -1,
-			   NULL,
-			   (GAsyncReadyCallback) ck_call_is_active_cb,
-			   application);
-
-        application->proxy = proxy;
-}
-
-static void
-ck_get_current_session_cb (GDBusConnection *connection,
-			   GAsyncResult    *result,
-			   gpointer         user_data)
-{
-	NautilusApplication *application = user_data;
-	GVariant *variant;
-	const char *session_path = NULL;
-	GError *error = NULL;
-
-	variant = g_dbus_connection_call_finish (connection, result, &error);
-
-	if (variant == NULL) {
-		g_warning ("Failed to get the current CK session: %s", error->message);
-		g_error_free (error);
-
-		application->session_is_active = TRUE;
-		return;
-	}
-
-	g_variant_get (variant, "(&o)", &session_path);
-
-	DEBUG ("Found ConsoleKit session at path %s", session_path);
-
-	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
-				  G_DBUS_PROXY_FLAGS_NONE,
-				  NULL,
-				  CK_NAME,
-				  session_path,
-				  CK_INTERFACE ".Session",
-				  NULL,
-				  session_proxy_appeared,
-				  application);
-
-	g_variant_unref (variant);
-}
-
-static void
-do_initialize_consolekit (NautilusApplication *application)
-{
-	GDBusConnection *connection;
-
-	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
-
-	if (connection == NULL) {
-		application->session_is_active = TRUE;
-		return;
-	}
-
-	g_dbus_connection_call (connection,
-				CK_NAME,
-				CK_PATH "/Manager",
-				CK_INTERFACE ".Manager",
-				"GetCurrentSession",
-				g_variant_new ("()"),
-				G_VARIANT_TYPE ("(o)"),
-				G_DBUS_CALL_FLAGS_NONE,
-				-1,
-				NULL,
-				(GAsyncReadyCallback) ck_get_current_session_cb,
-				application);
-
-	g_object_unref (connection);
-}
-
 static void
 do_upgrades_once (NautilusApplication *application,
 		  gboolean no_desktop)
@@ -479,33 +331,26 @@ finish_startup (NautilusApplication *application,
 	/* Initialize the desktop link monitor singleton */
 	nautilus_desktop_link_monitor_get ();
 
-	/* Initialize the ConsoleKit listener for active session */
-	do_initialize_consolekit (application);
-
-	/* Watch for mounts so we can restore open windows This used
-	 * to be for showing new window on mount, but is not used
-	 * anymore */
+	/* Initialize the UI handler singleton for file operations */
+	notify_init (GETTEXT_PACKAGE);
+	application->priv->progress_handler = nautilus_progress_ui_handler_new ();
 
 	/* Watch for unmounts so we can close open windows */
 	/* TODO-gio: This should be using the UNMOUNTED feature of GFileMonitor instead */
-	application->volume_monitor = g_volume_monitor_get ();
-	g_signal_connect_object (application->volume_monitor, "mount_removed",
+	application->priv->volume_monitor = g_volume_monitor_get ();
+	g_signal_connect_object (application->priv->volume_monitor, "mount_removed",
 				 G_CALLBACK (mount_removed_callback), application, 0);
-	g_signal_connect_object (application->volume_monitor, "mount_pre_unmount",
-				 G_CALLBACK (mount_removed_callback), application, 0);
-	g_signal_connect_object (application->volume_monitor, "mount_added",
+	g_signal_connect_object (application->priv->volume_monitor, "mount_added",
 				 G_CALLBACK (mount_added_callback), application, 0);
 }
 
 static void
 open_window (NautilusApplication *application,
 	     const char *startup_id,
-	     const char *uri, GdkScreen *screen, const char *geometry,
-	     gboolean browser_window)
+	     const char *uri, GdkScreen *screen, const char *geometry)
 {
 	GFile *location;
 	NautilusWindow *window;
-	gboolean existing, open_in_browser;
 
 	if (uri == NULL) {
 		location = g_file_new_for_path (g_get_home_dir ());
@@ -513,26 +358,11 @@ open_window (NautilusApplication *application,
 		location = g_file_new_for_uri (uri);
 	}
 
-	open_in_browser = browser_window ||
-		g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
-
-	DEBUG ("Opening new window at uri %s, browser %d", uri, open_in_browser);
-
-	existing = FALSE;
-
-	if (open_in_browser) {
-		window = nautilus_application_create_navigation_window (application,
-									startup_id,
-									screen);
-	} else {
-		window = nautilus_application_get_spatial_window (application,
-								  NULL,
-								  startup_id,
-								  location,
-								  screen,
-								  NULL);
-	}
+	DEBUG ("Opening new window at uri %s", uri);
 
+	window = nautilus_application_create_window (application,
+						     startup_id,
+						     screen);
 	nautilus_window_go_to (window, location);
 
 	g_object_unref (location);
@@ -555,18 +385,17 @@ open_windows (NautilusApplication *application,
 	      const char *startup_id,
 	      char **uris,
 	      GdkScreen *screen,
-	      const char *geometry,
-	      gboolean browser_window)
+	      const char *geometry)
 {
 	guint i;
 
 	if (uris == NULL || uris[0] == NULL) {
 		/* Open a window pointing at the default location. */
-		open_window (application, startup_id, NULL, screen, geometry, browser_window);
+		open_window (application, startup_id, NULL, screen, geometry);
 	} else {
 		/* Open windows at each requested location. */
 		for (i = 0; uris[i] != NULL; i++) {
-			open_window (application, startup_id, uris[i], screen, geometry, browser_window);
+			open_window (application, startup_id, uris[i], screen, geometry);
 		}
 	}
 }
@@ -673,15 +502,10 @@ nautilus_application_create_desktop_windows (NautilusApplication *application)
 	NautilusDesktopWindow *window;
 	GtkWidget *selection_widget;
 	int screens, i;
-	gboolean exit_with_last_window;
 
 	display = gdk_display_get_default ();
 	screens = gdk_display_get_n_screens (display);
 
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
-
 	for (i = 0; i < screens; i++) {
 
 		DEBUG ("Creating a desktop window for screen %d", i);
@@ -706,13 +530,8 @@ nautilus_application_create_desktop_windows (NautilusApplication *application)
 			nautilus_application_desktop_windows =
 				g_list_prepend (nautilus_application_desktop_windows, window);
 
-			/* don't add the desktop windows to the GtkApplication hold toplevels
-			 * if we should exit when the last window is closed.
-			 */
-			if (!exit_with_last_window) {
-				gtk_application_add_window (GTK_APPLICATION (application),
-							    GTK_WINDOW (window));
-			}
+			gtk_application_add_window (GTK_APPLICATION (application),
+						    GTK_WINDOW (window));
 		}
 	}
 }
@@ -737,7 +556,7 @@ nautilus_application_close_desktop (void)
 }
 
 void
-nautilus_application_close_all_navigation_windows (NautilusApplication *self)
+nautilus_application_close_all_windows (NautilusApplication *self)
 {
 	GList *list_copy;
 	GList *l;
@@ -748,144 +567,14 @@ nautilus_application_close_all_navigation_windows (NautilusApplication *self)
 		NautilusWindow *window;
 		
 		window = NAUTILUS_WINDOW (l->data);
-
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-			gtk_widget_hide (GTK_WIDGET (window));
-		}
+		gtk_widget_hide (GTK_WIDGET (window));
 	}
 
 	for (l = list_copy; l != NULL; l = l->next) {
 		NautilusWindow *window;
 		
 		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-			nautilus_window_close (window);
-		}
-	}
-	g_list_free (list_copy);
-}
-
-static NautilusSpatialWindow *
-nautilus_application_get_existing_spatial_window (GFile *location)
-{
-	GList *l;
-	NautilusWindowSlot *slot;
-	GFile *window_location;
-
-	for (l = nautilus_application_get_spatial_window_list ();
-	     l != NULL; l = l->next) {
-		slot = NAUTILUS_WINDOW (l->data)->details->active_pane->active_slot;
-
-		window_location = slot->pending_location;
-		
-		if (window_location == NULL) {
-			window_location = slot->location;
-		}
-
-		if (window_location != NULL) {
-			if (g_file_equal (location, window_location)) {
-				return NAUTILUS_SPATIAL_WINDOW (l->data);
-			}
-		}
-	}
-
-	return NULL;
-}
-
-static NautilusSpatialWindow *
-find_parent_spatial_window (NautilusSpatialWindow *window)
-{
-	NautilusFile *file;
-	NautilusFile *parent_file;
-	NautilusWindowSlot *slot;
-	GFile *location;
-
-	slot = NAUTILUS_WINDOW (window)->details->active_pane->active_slot;
-
-	location = slot->location;
-	if (location == NULL) {
-		return NULL;
-	}
-	file = nautilus_file_get (location);
-
-	if (!file) {
-		return NULL;
-	}
-
-	parent_file = nautilus_file_get_parent (file);
-	nautilus_file_unref (file);
-	while (parent_file) {
-		NautilusSpatialWindow *parent_window;
-
-		location = nautilus_file_get_location (parent_file);
-		parent_window = nautilus_application_get_existing_spatial_window (location);
-		g_object_unref (location);
-
-		/* Stop at the desktop directory if it's not explicitely opened
-		 * in a spatial window of its own.
-		 */
-		if (nautilus_file_is_desktop_directory (parent_file) && !parent_window) {
-			nautilus_file_unref (parent_file);
-			return NULL;
-		}
-
-		if (parent_window) {
-			nautilus_file_unref (parent_file);
-			return parent_window;
-		}
-		file = parent_file;
-		parent_file = nautilus_file_get_parent (file);
-		nautilus_file_unref (file);
-	}
-
-	return NULL;
-}
-
-void
-nautilus_application_close_parent_windows (NautilusSpatialWindow *window)
-{
-	NautilusSpatialWindow *parent_window;
-	NautilusSpatialWindow *new_parent_window;
-
-	g_return_if_fail (NAUTILUS_IS_SPATIAL_WINDOW (window));
-
-	parent_window = find_parent_spatial_window (window);
-	
-	while (parent_window) {
-		
-		new_parent_window = find_parent_spatial_window (parent_window);
-		nautilus_window_close (NAUTILUS_WINDOW (parent_window));
-		parent_window = new_parent_window;
-	}
-}
-
-void
-nautilus_application_close_all_spatial_windows (void)
-{
-	GList *list_copy;
-	GList *l;
-	
-	list_copy = g_list_copy (nautilus_application_spatial_window_list);
-	/* First hide all window to get the feeling of quick response */
-	for (l = list_copy; l != NULL; l = l->next) {
-		NautilusWindow *window;
-		
-		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			gtk_widget_hide (GTK_WIDGET (window));
-		}
-	}
-
-	for (l = list_copy; l != NULL; l = l->next) {
-		NautilusWindow *window;
-		
-		window = NAUTILUS_WINDOW (l->data);
-		
-		if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-			nautilus_window_close (window);
-		}
+		nautilus_window_close (window);
 	}
 	g_list_free (list_copy);
 }
@@ -906,7 +595,6 @@ nautilus_window_delete_event_callback (GtkWidget *widget,
 
 static NautilusWindow *
 create_window (NautilusApplication *application,
-	       GType window_type,
 	       const char *startup_id,
 	       GdkScreen *screen)
 {
@@ -914,10 +602,10 @@ create_window (NautilusApplication *application,
 	
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	
-	window = NAUTILUS_WINDOW (gtk_widget_new (window_type,
-						  "app", application,
-						  "screen", screen,
-						  NULL));
+	window = g_object_new (NAUTILUS_TYPE_WINDOW,
+			       "app", application,
+			       "screen", screen,
+			       NULL);
 
 	if (startup_id) {
 		gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
@@ -938,75 +626,6 @@ create_window (NautilusApplication *application,
 	return window;
 }
 
-static void
-spatial_window_destroyed_callback (void *user_data, GObject *window)
-{
-	nautilus_application_spatial_window_list =
-		g_list_remove (nautilus_application_spatial_window_list, window);
-}
-
-NautilusWindow *
-nautilus_application_get_spatial_window (NautilusApplication *application,
-					 NautilusWindow      *requesting_window,
-					 const char          *startup_id,
-					 GFile               *location,
-					 GdkScreen           *screen,
-					 gboolean            *existing)
-{
-	NautilusWindow *window;
-
-	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
-
-	window = NAUTILUS_WINDOW
-		(nautilus_application_get_existing_spatial_window (location));
-
-	if (window != NULL) {
-		if (existing != NULL) {
-			*existing = TRUE;
-		}
-
-		DEBUG ("returning existing spatial window");
-
-		return window;
-	}
-
-	if (existing != NULL) {
-		*existing = FALSE;
-	}
-
-	window = create_window (application, NAUTILUS_TYPE_SPATIAL_WINDOW, startup_id, screen);
-	if (requesting_window) {
-		/* Center the window over the requesting window by default */
-		int orig_x, orig_y, orig_width, orig_height;
-		int new_x, new_y, new_width, new_height;
-		
-		gtk_window_get_position (GTK_WINDOW (requesting_window), 
-					 &orig_x, &orig_y);
-		gtk_window_get_size (GTK_WINDOW (requesting_window), 
-				     &orig_width, &orig_height);
-		gtk_window_get_default_size (GTK_WINDOW (window),
-					     &new_width, &new_height);
-		
-		new_x = orig_x + (orig_width - new_width) / 2;
-		new_y = orig_y + (orig_height - new_height) / 2;
-		
-		if (orig_width - new_width < 10) {
-			new_x += 10;
-			new_y += 10;
-		}
-
-		gtk_window_move (GTK_WINDOW (window), new_x, new_y);
-	}
-
-	nautilus_application_spatial_window_list = g_list_prepend (nautilus_application_spatial_window_list, window);
-	g_object_weak_ref (G_OBJECT (window), 
-			   spatial_window_destroyed_callback, NULL);
-
-	DEBUG ("Creating new spatial window");
-
-	return window;
-}
-
 static gboolean
 another_navigation_window_already_showing (NautilusApplication *application,
 					   NautilusWindow *the_window)
@@ -1015,8 +634,7 @@ another_navigation_window_already_showing (NautilusApplication *application,
 	
 	list = gtk_application_get_windows (GTK_APPLICATION (application));
 	for (item = list; item != NULL; item = item->next) {
-		if (item->data != the_window &&
-		    NAUTILUS_IS_NAVIGATION_WINDOW (item->data)) {
+		if (item->data != the_window) {
 			return TRUE;
 		}
 	}
@@ -1025,9 +643,9 @@ another_navigation_window_already_showing (NautilusApplication *application,
 }
 
 NautilusWindow *
-nautilus_application_create_navigation_window (NautilusApplication *application,
-					       const char          *startup_id,
-					       GdkScreen           *screen)
+nautilus_application_create_window (NautilusApplication *application,
+				    const char          *startup_id,
+				    GdkScreen           *screen)
 {
 	NautilusWindow *window;
 	char *geometry_string;
@@ -1035,7 +653,7 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 
-	window = create_window (application, NAUTILUS_TYPE_NAVIGATION_WINDOW, startup_id, screen);
+	window = create_window (application, startup_id, screen);
 
 	maximized = g_settings_get_boolean
 		(nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
@@ -1056,8 +674,8 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 		eel_gtk_window_set_initial_geometry_from_string 
 			(GTK_WINDOW (window), 
 			 geometry_string,
-			 NAUTILUS_NAVIGATION_WINDOW_MIN_WIDTH,
-			 NAUTILUS_NAVIGATION_WINDOW_MIN_HEIGHT,
+			 NAUTILUS_WINDOW_MIN_WIDTH,
+			 NAUTILUS_WINDOW_MIN_HEIGHT,
 			 another_navigation_window_already_showing (application, window));
 	}
 	g_free (geometry_string);
@@ -1099,10 +717,6 @@ mount_added_callback (GVolumeMonitor *monitor,
 	NautilusDirectory *directory;
 	GFile *root;
 	gchar *uri;
-
-	if (!application->session_is_active) {
-		return;
-	}
 		
 	root = g_mount_get_root (mount);
 	uri = g_file_get_uri (root);
@@ -1124,9 +738,7 @@ get_first_navigation_slot (GList *slot_list)
 	GList *l;
 
 	for (l = slot_list; l != NULL; l = l->next) {
-		if (NAUTILUS_IS_NAVIGATION_WINDOW_SLOT (l->data)) {
-			return l->data;
-		}
+		return l->data;
 	}
 
 	return NULL;
@@ -1138,19 +750,13 @@ should_close_slot_with_mount (NautilusWindow *window,
 			      NautilusWindowSlot *slot,
 			      GMount *mount)
 {
-	if (NAUTILUS_IS_SPATIAL_WINDOW (window)) {
-		return TRUE;
-	}
-	return nautilus_navigation_window_slot_should_close_with_mount (NAUTILUS_NAVIGATION_WINDOW_SLOT (slot),
-									mount);
+	return nautilus_window_slot_should_close_with_mount (slot, mount);
 }
 
 /* Called whenever a mount is unmounted. Check and see if there are
  * any windows open displaying contents on the mount. If there are,
  * close them.  It would also be cool to save open window and position
  * info.
- *
- * This is also called on pre_unmount.
  */
 static void
 mount_removed_callback (GVolumeMonitor *monitor,
@@ -1222,9 +828,9 @@ mount_removed_callback (GVolumeMonitor *monitor,
 
 		if (should_close_slot_with_mount (window, slot, mount) &&
 		    slot != force_no_close_slot) {
-			nautilus_window_slot_close (slot);
+			nautilus_window_pane_slot_close (slot->pane, slot);
 		} else {
-			computer = g_file_new_for_uri ("computer:///");
+			computer = g_file_new_for_path (g_get_home_dir ());
 			nautilus_window_slot_go_to (slot, computer, FALSE);
 			g_object_unref(computer);
 		}
@@ -1256,7 +862,9 @@ nautilus_application_constructor (GType type,
 static void
 nautilus_application_init (NautilusApplication *application)
 {
-	/* do nothing */
+	application->priv =
+		G_TYPE_INSTANCE_GET_PRIVATE (application, NAUTILUS_TYPE_APPLICATION,
+					     NautilusApplicationPriv);
 }
 
 static void
@@ -1268,22 +876,12 @@ nautilus_application_finalize (GObject *object)
 
 	nautilus_bookmarks_exiting ();
 
-	if (application->undo_manager != NULL) {
-		g_object_unref (application->undo_manager);
-		application->undo_manager = NULL;
-	}
-
-	if (application->volume_monitor) {
-		g_object_unref (application->volume_monitor);
-		application->volume_monitor = NULL;
-	}
-
-	if (application->proxy != NULL) {
-		g_object_unref (application->proxy);
-		application->proxy = NULL;
-	}
+	g_clear_object (&application->undo_manager);
+	g_clear_object (&application->priv->volume_monitor);
+	g_clear_object (&application->priv->progress_handler);
 
 	nautilus_dbus_manager_stop ();
+	notify_uninit ();
 
         G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
@@ -1292,18 +890,10 @@ void
 nautilus_application_quit (NautilusApplication *self)
 {
 	GApplication *app = G_APPLICATION (self);
-	gboolean exit_with_last_window;
+	GList *windows;
 
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
-
-	nautilus_application_close_desktop ();
-	g_application_release (app);
-
-	if (!exit_with_last_window) {
-		g_application_release (app);
-	}
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
 }
 
 static gint
@@ -1314,7 +904,6 @@ nautilus_application_command_line (GApplication *app,
 	gboolean version = FALSE;
 	gboolean no_default_window = FALSE;
 	gboolean no_desktop = FALSE;
-	gboolean browser_window = FALSE;
 	gboolean kill_shell = FALSE;
 	gboolean autostart_mode = FALSE;
 	const gchar *autostart_id;
@@ -1333,8 +922,6 @@ nautilus_application_command_line (GApplication *app,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
 		{ "no-desktop", '\0', 0, G_OPTION_ARG_NONE, &no_desktop,
 		  N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
-		{ "browser", '\0', 0, G_OPTION_ARG_NONE, &browser_window, 
-		  N_("Open a browser window."), NULL },
 		{ "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
 		  N_("Quit Nautilus."), NULL },
 		{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining, NULL,  N_("[URI...]") },
@@ -1347,21 +934,13 @@ nautilus_application_command_line (GApplication *app,
 	gint argc = 0;
 	gchar **argv = NULL, **uris = NULL;
 	gint retval = EXIT_SUCCESS;
-	gboolean exit_with_last_window;
 
 	context = g_option_context_new (_("\n\nBrowse the file system with the file manager"));
 	g_option_context_add_main_entries (context, options, NULL);
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
-	if (!self->initialized) {
-		g_option_context_add_group (context, egg_sm_client_get_option_group ());
-	}
-
 	argv = g_application_command_line_get_arguments (command_line, &argc);
 
-	/* we need to do this here, as parsing the EggSMClient option context,
-	 * unsets this variable.
-	 */
 	autostart_id = g_getenv ("DESKTOP_AUTOSTART_ID");
 	if (autostart_id != NULL && *autostart_id != '\0') {
 		autostart_mode = TRUE;
@@ -1387,7 +966,7 @@ nautilus_application_command_line (GApplication *app,
 	}
 	if (kill_shell && remaining != NULL) {
 		g_application_command_line_printerr (command_line, "%s\n",
-						     _("-- quit cannot be used with URIs."));
+						     _("--quit cannot be used with URIs."));
 		retval = EXIT_FAILURE;
 		goto out;
 	}
@@ -1433,18 +1012,14 @@ nautilus_application_command_line (GApplication *app,
 		no_desktop = FALSE;
 	}
 
-	DEBUG ("Parsing command line, no_default_window %d browser %d, quit %d, "
+	DEBUG ("Parsing command line, no_default_window %d, quit %d, "
 	       "self checks %d, no_desktop %d",
-	       no_default_window, browser_window, kill_shell, perform_self_check, no_desktop);
-
-	exit_with_last_window =
-		g_settings_get_boolean (nautilus_preferences,
-					NAUTILUS_PREFERENCES_EXIT_WITH_LAST_WINDOW);
+	       no_default_window, kill_shell, perform_self_check, no_desktop);
 
 	if (kill_shell) {
 		nautilus_application_quit (self);
 	} else {
-		if (!self->initialized) {
+		if (!self->priv->initialized) {
 			char *accel_map_filename;
 
 			if (!no_desktop &&
@@ -1457,10 +1032,6 @@ nautilus_application_command_line (GApplication *app,
 				nautilus_application_open_desktop (self);
 			}
 
-			if (!exit_with_last_window) {
-				g_application_hold (app);
-			}
-
 			finish_startup (self, no_desktop);
 
 			/* Monitor the preference to show or hide the desktop */
@@ -1478,10 +1049,7 @@ nautilus_application_command_line (GApplication *app,
 			g_signal_connect (gtk_accel_map_get (), "changed",
 					  G_CALLBACK (queue_accel_map_save_callback), NULL);
 
-			/* Initialize SMClient and load session info if availible */
-			nautilus_application_smclient_load (self, &no_default_window);
-
-			self->initialized = TRUE;
+			self->priv->initialized = TRUE;
 		}
 
 		/* Convert args to URIs */
@@ -1514,8 +1082,7 @@ nautilus_application_command_line (GApplication *app,
 			open_windows (self, NULL,
 				      uris,
 				      gdk_screen_get_default (),
-				      geometry,
-				      browser_window);
+				      geometry);
 		}
 	}
 
@@ -1526,6 +1093,28 @@ nautilus_application_command_line (GApplication *app,
 	return retval;
 }
 
+static void
+init_css (void)
+{
+	GtkCssProvider *provider;
+	GError *error = NULL;
+
+	provider = gtk_css_provider_new ();
+	gtk_css_provider_load_from_path (provider,
+					 NAUTILUS_DATADIR G_DIR_SEPARATOR_S "nautilus.css", &error);
+
+	if (error != NULL) {
+		g_warning ("Can't parse Nautilus' CSS custom description: %s\n", error->message);
+		g_error_free (error);
+	} else {
+		gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+							   GTK_STYLE_PROVIDER (provider),
+							   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	}
+
+	g_object_unref (provider);
+}
+
 static void
 nautilus_application_startup (GApplication *app)
 {
@@ -1541,31 +1130,31 @@ nautilus_application_startup (GApplication *app)
 	/* create an undo manager */
 	self->undo_manager = nautilus_undo_manager_new ();
 
-	/* initialize the session manager client */
-	nautilus_application_smclient_startup (self);
-
 	/* Initialize preferences. This is needed to create the
 	 * global GSettings objects.
 	 */
 	nautilus_global_preferences_init ();
 
 	/* register views */
-	fm_icon_view_register ();
-	fm_desktop_icon_view_register ();
-	fm_list_view_register ();
-	fm_compact_view_register ();
+	nautilus_icon_view_register ();
+	nautilus_desktop_icon_view_register ();
+	nautilus_list_view_register ();
+	nautilus_icon_view_compact_register ();
 #if ENABLE_EMPTY_VIEW
-	fm_empty_view_register ();
+	nautilus_empty_view_register ();
 #endif /* ENABLE_EMPTY_VIEW */
 
 	/* register property pages */
 	nautilus_image_properties_page_register ();
 
+	/* initialize CSS theming */
+	init_css ();
+
 	/* initialize search path for custom icons */
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
 					   NAUTILUS_DATADIR G_DIR_SEPARATOR_S "icons");
 
-	nautilus_dbus_manager_start ();
+	nautilus_dbus_manager_start (app);
 }
 
 static void
@@ -1593,6 +1182,8 @@ nautilus_application_class_init (NautilusApplicationClass *class)
 	application_class->startup = nautilus_application_startup;
 	application_class->command_line = nautilus_application_command_line;
 	application_class->quit_mainloop = nautilus_application_quit_mainloop;
+
+	g_type_class_add_private (class, sizeof (NautilusApplication));
 }
 
 NautilusApplication *
